
import os
import shutil
import utils

##############################################################################
# Template
##############################################################################

def get_templates(package):
    template_common_dir = os.path.join(os.path.dirname(__file__),'templates','common') 
    template_dir = os.path.join(os.path.dirname(__file__),'templates','cpp-ros') 
    templates = {}
    templates['mainpage.dox'] = utils.read_template(os.path.join(template_common_dir,'mainpage.dox'))
    templates['manifest.xml'] = utils.read_template(os.path.join(template_common_dir,'manifest.xml'))
    templates['CMakeLists.txt'] = utils.read_template(os.path.join(template_dir,'CMakeLists.txt'))
    templates[os.path.join('src','main.cpp')] = utils.read_template(os.path.join(template_dir,'src','main.cpp'))
    templates[os.path.join('src','CMakeLists.txt')] = utils.read_template(os.path.join(template_dir,'src','CMakeLists.txt'))
    templates[os.path.join('src','lib',package+'.cpp')] = utils.read_template(os.path.join(template_dir,'src','lib','package_name.cpp'))
    templates[os.path.join('src','lib','CMakeLists.txt')] = utils.read_template(os.path.join(template_dir,'src','lib','CMakeLists.txt'))
    templates[os.path.join('include',package,package+'.hpp')] = utils.read_template(os.path.join(template_dir,'include','PACKAGE_NAME','package_name.hpp'))
    templates[os.path.join('launch','test.launch')] = utils.read_template(os.path.join(template_dir,'launch','test.launch'))
    return templates

def create_cpp_ros_package():

    (package, depends) = utils.parse_arguments(['roscpp','std_msgs'])
    # Make directories
    p = os.path.abspath(package)
    os.makedirs(os.path.join(p,"src"))
    print "Created package directory 'src'"
    os.makedirs(os.path.join(p,"src","lib"))
    print "Created package directory 'src/lib'"
    os.makedirs(os.path.join(p,"include"))
    print "Created package directory 'include'"
    os.makedirs(os.path.join(p,"include",package))
    print "Created package directory 'include/%s'"%package
    os.makedirs(os.path.join(p,"launch"))
    print "Created package directory 'launch'"

    # Text files
    manifest_depends = ''.join(['  <depend package="%s"/>\n'%d for d in depends])
    cmake_depends = ''.join(['%s '%d for d in depends])
    templates = get_templates(package)
    for filename, template in templates.iteritems():
        contents = utils.instantiate_template(template, package, package, package, utils.author_name(), manifest_depends, cmake_depends)
        try:
            p = os.path.abspath(os.path.join(package, filename))
            f = open(p, 'w')
            f.write(contents.encode('utf-8'))
            print "Created package file", p
        finally:
            f.close()
    utils.print_concluding_catkin_message(package)
    




__version__ = '0.1.8'

from .qtcreate import create_qt_ros_package
from .roscreate import create_ros_package
from .cppcreate import create_cpp_package
from .cpproscreate import create_cpp_ros_package
from .commscreate import create_comms_package
from .stackcreate import create_stack




import os
import shutil
import utils

##############################################################################
# Template
##############################################################################

def get_text_templates():
    template_dir = os.path.join(os.path.dirname(__file__),'templates','stack') 
    templates = {}
    templates['CMakeLists.txt'] = utils.read_template(os.path.join(template_dir,'CMakeLists.txt'))
    templates['stack.xml'] = utils.read_template(os.path.join(template_dir,'stack.xml'))
    return templates

def create_stack():
    
    (stack, depends) = utils.parse_stack_arguments()

    s = os.path.abspath(stack)
    os.makedirs(s) 
    manifest_depends = ''.join(['  <build_depends>%s</build_depends>\n'%d for d in depends])
    cmake_depends = [] # These are currently a bit odd, they're packages
    templates = get_text_templates()
    for filename, template in templates.iteritems():
        contents = utils.instantiate_template(template, stack, stack, stack, utils.author_name(), manifest_depends, cmake_depends)
        try:
            p = os.path.abspath(os.path.join(stack, filename))
            f = open(p, 'w')
            f.write(contents.encode('utf-8'))
            print "Created stack file", p
        finally:
            f.close()




from distutils.core import setup

import sys
sys.path.insert(0, 'src')

from roscreate2 import __version__

setup(name='roscreate',
      version= __version__,
      packages=['roscreate2'],
      package_dir = {'':'src'},
      scripts = ["scripts/roscreate-pkg","scripts/roscreate-catkin-pkg","scripts/roscreate-legacy-pkg", "scripts/roscreate-comms-pkg", "scripts/roscreate-qt-pkg","scripts/roscreate-qt-legacy-pkg","scripts/roscreate-stack","scripts/roscreate-cpp-pkg","scripts/roscreate-cpp-ros-pkg",
	             "scripts/roscreate-pkg.bat","scripts/roscreate-catkin-pkg.bat","scripts/roscreate-comms-pkg.bat", "scripts/roscreate-qt-pkg.bat","scripts/roscreate-stack.bat","scripts/roscreate-cpp-pkg.bat","scripts/roscreate-cpp-ros-pkg.bat"],
      package_data = {'roscreate2': [
           'templates/common/mainpage.dox',
           'templates/common/Makefile',
           'templates/common/manifest.xml',
           'templates/qt-ros/CMakeLists.txt',
           'templates/qt-ros/src/*.cpp',
           'templates/qt-ros/ui/*.ui',
           'templates/qt-ros/include/PACKAGE_NAME/*.hpp',
           'templates/qt-ros/resources/*.qrc',
           'templates/qt-ros/resources/images/*.png',
           'templates/qt-ros/manifest.xml',
           'templates/qt-ros-legacy/CMakeLists.txt',
           'templates/qt-ros-legacy/src/*.cpp',
           'templates/qt-ros-legacy/ui/*.ui',
           'templates/qt-ros-legacy/include/PACKAGE_NAME/*.hpp',
           'templates/qt-ros-legacy/resources/*.qrc',
           'templates/qt-ros-legacy/resources/images/*.png',
           'templates/qt-ros-legacy/manifest.xml',
           'templates/ros/CMakeLists.txt',
           'templates/ros/manifest.xml',
           'templates/cpp/CMakeLists.txt',
           'templates/cpp/manifest.xml',
           'templates/cpp/include/PACKAGE_NAME/package_name.hpp',
           'templates/cpp/src/main.cpp',
           'templates/cpp/src/CMakeLists.txt',
           'templates/cpp/src/lib/package_name.cpp',
           'templates/cpp/src/lib/CMakeLists.txt',
           'templates/cpp-ros/CMakeLists.txt',
           'templates/cpp-ros/manifest.xml',
           'templates/cpp-ros/include/PACKAGE_NAME/package_name.hpp',
           'templates/cpp-ros/src/main.cpp',
           'templates/cpp-ros/src/CMakeLists.txt',
           'templates/cpp-ros/src/lib/package_name.cpp',
           'templates/cpp-ros/src/lib/CMakeLists.txt',
           'templates/cpp-ros/launch/test.launch',
           'templates/ros-legacy/CMakeLists.txt',
           'templates/ros-legacy/manifest.xml',
           'templates/comms/CMakeLists.txt',
           'templates/comms/manifest.xml',
           'templates/comms/msg/Dude.msg',
           'templates/comms/srv/HelloDude.srv',
           'templates/stack/CMakeLists.txt',
           'templates/stack/stack.xml',
           ]},
      author = "Daniel Stonier",
      author_email = "d.stonier@gmail.com",
      url = "http://pypi.python.org/pypi/roscreate/",
      download_url = "https://github.com/yujinrobot/roscreate.git",
      keywords = ["ROS"],
      classifiers = [
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License" ],
      description = "Creational templates for RoS",
      long_description = "Refer to the documentation at https://github.com/yujinrobot/roscreate.",
      license = "BSD"
      )

# This no longer works...it has to be a special parsable syntax of pypi's.
#      long_description = open('README.md').read(),




##############################################################################
# Utilities
##############################################################################

import os
import re
from optparse import OptionParser
import rospkg

##################
# Author
##################

def author_name():
    """
    Utility to compute logged in user name
    
    :returns: name of current user, ``str``
    """
    import getpass
    name = getpass.getuser()
    try:
        import pwd
        login = name
        name = pwd.getpwnam(login)[4]
        name = ''.join(name.split(',')) # strip commas
        # in case pwnam is not set
        if not name:
            name = login
    except:
        #pwd failed
        pass
    if type(name) == str:
        name = name.decode('utf-8')
    return name

##################
# Templates
##################

# Finds and reads one of the templates.
def read_template(tmplf):
    f = open(tmplf, 'r')
    try:
        t = f.read()
    finally:
        f.close()
    return t

# This inserts the labelled variables into the template wherever the corresponding
# %package, %brief, %description %manifest_depends and %cmake_depends is found.
def instantiate_template(template, name, brief, description, author, manifest_depends, cmake_depends):
    return template%locals()


##################
# Names
##################

# These routines are originally from roslib
# Todo: might be nice if they were in rospkg, not here.

BASE_RESOURCE_NAME_LEGAL_CHARS_P = re.compile('^[A-Za-z][\w_]*$') #ascii char followed by (alphanumeric, _)
def is_legal_resource_base_name(name):
    """
    Validates that name is a legal resource base name. A base name has
    no package context, e.g. "String".
    """
    # resource names can be unicode due to filesystem
    if name is None:
        return False
    m = BASE_RESOURCE_NAME_LEGAL_CHARS_P.match(name)
    return m is not None and m.group(0) == name

##################
# Argument Parser
##################

def parse_arguments(extra_depends=[]):
    """
    Parse the command line arguments - in format <package-name> [dependencies]".
    
    This will need to upgrade from optparse to argparse post python 2.7 
    """
    parser = OptionParser(usage="usage: %prog <package-name> [dependencies...]")
    options, args = parser.parse_args()
    if not args:
        parser.error("You must specify a package name and optionally also list package dependencies")
    package = args[0]
    if type(package) == str:
        package = package.decode('utf-8')
    depends = args[1:] + extra_depends
    if not is_legal_resource_base_name(package):
        parser.error("illegal package name [%s]\n\nNames must start with a letter and contain only alphanumeric characters\nand underscores."%package)
    if os.path.exists(os.path.abspath(package)):
        parser.error("%s already exists, aborting"%package)
    if not rospkg.get_ros_paths():
        # Common in a catkin environment - you often don't sit in build's env.sh while working.
        print("WARNING: no ros environment found, skipping package and dependency checks\n")
    else:
        rospack = rospkg.RosPack()
        if package_already_exists(package, rospack): 
            parser.error("package already exists on the ROS_PACKAGE_PATH!")    
        if not rospkg.on_ros_path(package):
            parser.error("current working directory is not on ROS_PACKAGE_PATH!\nPlease update your ROS_PACKAGE_PATH environment variable.\n")    
        if not dependencies_exist(depends, rospack):
           parser.error("dependency [%s] cannot be found"%d)
    return (package,depends)

def parse_stack_arguments():
    """
    Parse the command line arguments - in format <stack-name> [dependencies]".
    
    This will need to upgrade from optparse to argparse post python 2.7 
    """
    parser = OptionParser(usage="usage: %prog <stack-name> [stack dependencies...]")
    options, args = parser.parse_args()
    if not args:
        parser.error("You must specify a stack name and optionally also list stack dependencies")
    stack = args[0]
    if type(stack) == str:
        stack = stack.decode('utf-8')
    depends = args[1:]
    if not is_legal_resource_base_name(stack):
        parser.error("illegal stack name [%s]\n\nNames must start with a letter and contain only alphanumeric characters\nand underscores."%package)
    if os.path.exists(os.path.abspath(stack)):
        parser.error("%s already exists, aborting"%stack)
    # Could probably check if the stack already exists somewhere first, also dependencies.
    # We'd need rosstack though
    #if not rospkg.get_ros_paths():
        # Common in a catkin environment - you often don't sit in build's env.sh while working.
    #    print("WARNING: no ros environment found, skipping package and dependency checks\n")
    #else:
    #    rospack = rospkg.RosPack()
    #    if package_already_exists(package, rospack): 
    #        parser.error("package already exists on the ROS_PACKAGE_PATH!")    
    #    if not rospkg.on_ros_path(package):
    #        parser.error("current working directory is not on ROS_PACKAGE_PATH!\nPlease update your ROS_PACKAGE_PATH environment variable.\n")    
    #    if not dependencies_exist(depends, rospack):
    #       parser.error("dependency [%s] cannot be found"%d)
    return (stack,depends)

##################
# Checks
##################

def package_already_exists(package, rospack=rospkg.RosPack()):
    try:
        rospack.get_path(package)
    except rospkg.ResourceNotFound:
        return False
    return True
    
def dependencies_exist(depends, rospack=rospkg.RosPack()):
    for d in depends:
        try:
            rospack.get_path(d)
        except rospkg.ResourceNotFound:
            return False
    return True

##################
# Messages
##################

def print_concluding_message(package):
    print "\nPlease edit %s/manifest.xml and mainpage.dox to finish creating your package"%package

def print_concluding_catkin_message(package):
    print "\n***** Important - Update your Stack Info *****\n  1) edit the stack CMakeLists.txt file and add the package subdirectory.\n  2) edit the stack.xml and add your <build_depends> and <depends> tags\n"





import os
import utils

##############################################################################
# Template
##############################################################################

def get_text_templates(package, type):
    template_common_dir = os.path.join(os.path.dirname(__file__),'templates','common') 
    template_dir = os.path.join(os.path.dirname(__file__),'templates',type) 
    templates = {}
    templates['mainpage.dox'] = utils.read_template(os.path.join(template_common_dir,'mainpage.dox'))
    templates['CMakeLists.txt'] = utils.read_template(os.path.join(template_dir,'CMakeLists.txt'))
    templates['manifest.xml'] = utils.read_template(os.path.join(template_dir,'manifest.xml'))
    templates[os.path.join('msg','Dude.msg')] = utils.read_template(os.path.join(template_dir,'msg','Dude.msg'))
    templates[os.path.join('srv','HelloDude.srv')] = utils.read_template(os.path.join(template_dir,'srv','HelloDude.srv'))
    return templates

def create_comms_package(type):
    
    (package, depends) = utils.parse_arguments(['std_msgs'])
    # Make directories
    p = os.path.abspath(package)
    os.makedirs(os.path.join(p,"msg"))
    os.makedirs(os.path.join(p,"srv"))
    print "Created package directories."

    # Text files
    manifest_depends = ''.join(['  <depend package="%s"/>\n'%d for d in depends])
    cmake_depends = ''.join(['%s '%d for d in depends])
    templates = get_text_templates(package, type)
    for filename, template in templates.iteritems():
        contents = utils.instantiate_template(template, package, package, package, utils.author_name(), manifest_depends, cmake_depends)
        try:
            p = os.path.abspath(os.path.join(package, filename))
            f = open(p, 'w')
            f.write(contents.encode('utf-8'))
            print "Created package file", p
        finally:
            f.close()
    utils.print_concluding_catkin_message(package)
    
